--[[
this lib adds a scanning tooltip to get names of quests by ID, something the wow api does not
natively support.
for now, this lib will behave rather "strict", e.g. update and get will not automatically add
the quest and so on.
--]]

if(ThingsToDo2 == nil) then
	ThingsToDo2 = {}
end
local ttd2 = ThingsToDo2

if(ttd2.lib == nil) then
	ttd2.lib = {}
end
local lib = ttd2.lib

--this module
lib.quest = {}
local this = lib.quest

this.private = { set = false }

function this.private.Validate()
	if(this.private.set == false) then
		this.private.Init()
	end
	return
end

function this.private.Init()
	if(this.private.set) then
		return
	end
	
	this.private.data = {}
	this.private.incomplete = {}
	
	this.private.set = true
end

function this.private.ConstructLink(ID, Name)
	this.private.Validate()
	
	if(Name == nil) then
		Name = "quest:"..ID
	end
	
	--TODO: currently level 110 is hardcoded, may be a problem later
	local LinkParts = {}
	LinkParts[1] = "\124cFFFFFF00\124Hquest:"
	LinkParts[2] = ":110\124h["
	LinkParts[3] = "]\124h\124r"
	
	return (LinkParts[1]..ID..LinkParts[2]..Name..LinkParts[3])
end

function this.AddByID(ID, StoreObjectives)
	this.private.Validate()
	
	--adding the same quest multiple times must be allowed because different modules might do it
	if(this.private.data[ID] ~= nil) then
		if(StoreObjectives == true and this.private.data[ID].Objectives == nil) then
			this.private.data[ID].Objectives = {}
		end
		return
	end
	
	this.private.data[ID] = 
	{
		Name = nil,
		Link = nil,
		IsCompleted = nil,
		IsInLog = nil
	}
	
	if(StoreObjectives == true) then
		this.private.data[ID].Objectives = {}
	end
	
	this.private.incomplete[ID] = true
	this.UpdateByID(ID)
	return
end

function this.UpdateByID(ID)
	this.private.Validate()
	
	if(this.private.data[ID] == nil) then
		lib.chat.error("trying to update quest before adding it: "..ID)
		return
	end
	
	if(this.private.incomplete[ID] ~= nil) then
		local complete = true
		
		lib.tooltip.RequestQuestByID(ID)
		local name = nil
		if(lib.tooltip.IsReady() == true) then
			name = lib.tooltip.GetTextLines()[1]
		end
			
		if((name == nil) or (name == " ")) then --there is currently a bug with empty questnames
			this.private.data[ID].Name = ("quest:"..ID)
			complete = false
		else
			this.private.data[ID].Name = name
		end
	
		this.private.data[ID].Link = this.private.ConstructLink(ID, this.private.data[ID].Name)
		
		if(complete) then
			this.private.incomplete[ID] = nil
		end
	end
	
	--this can change and always has to be updated
	local LogIndex = GetQuestLogIndexByID(ID)
	this.private.data[ID].IsCompleted = IsQuestFlaggedCompleted(ID)
	this.private.data[ID].IsInLog = (LogIndex ~= 0)
	this.private.data[ID].IsTask = IsQuestTask(ID)
	this.private.data[ID].TimeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(ID) --this API also workds for certain non-task quests; most notably: Emissaries
	if(this.private.data[ID].TimeLeftMinutes) then
		this.private.data[ID].TimeLeftString = "("..string.format(BLACK_MARKET_HOT_ITEM_TIME_LEFT,string.format(SPELL_DURATION_HOURS ,this.private.data[ID].TimeLeftMinutes/60.0))..")"
	else
		this.private.data[ID].TimeLeftString = " " --provide a space to prevent nil-concat errors during loading
	end
	
	--if objectives were not requested when adding the quest, we can end here
	if(this.private.data[ID].Objectives == nil) then
		return
	end
	
	--now lets get some objectives...
	if(this.private.data[ID].IsCompleted == true) then
		--if a quest is completed, it does not have objectives; clear the table if it was used before
		lib.table.clear(this.private.data[ID].Objectives)
	elseif(this.private.data[ID].IsInLog == true) then
		--quest is not compledted and is in the log. lets get some data.
		local NumObjectives = GetNumQuestLeaderBoards(LogIndex)
		if(NumObjectives == 0) then
			lib.table.clear(this.private.data[ID].Objectives)
		else
			lib.table.clear(this.private.data[ID].Objectives)
			for i=1,NumObjectives do
				local ObjectiveText, ObjectiveType, ObjectiveFinished = GetQuestLogLeaderBoard(i,LogIndex)
				this.private.data[ID].Objectives[i] = 
				{
					Text		= ObjectiveText,
					Type		= ObjectiveType,
					IsCompleted	= ObjectiveFinished
				}
			end
		end		
	elseif(this.private.data[ID].IsTask == true) then
		--for task quests there is an alternative API
		
		local _,_,NumObjectives = GetTaskInfo(ID)
		if(NumObjectives == 0) then
			lib.table.clear(this.private.data[ID].Objectives)
		else
			lib.table.clear(this.private.data[ID].Objectives)
			for i=1,NumObjectives do
				local ObjectiveText, ObjectiveType, ObjectiveFinished = GetQuestObjectiveInfo(ID,Index,false)
				this.private.data[ID].Objectives[i] = 
				{
					Text		= ObjectiveText,
					Type		= ObjectiveType,
					IsCompleted	= ObjectiveFinished
				}
			end
		end		
		lib.table.clear(this.private.data[ID].Objectives)
	elseif(this.private.data[ID].IsInLog == false) then
		--quest in not complete, not in log and not a task - nothing we can say about objectives
		lib.table.clear(this.private.data[ID].Objectives)
	else
		lib.chat.debug(this.name..": encountered unexpected quest status")
	end

	return
end

function this.GetByID(ID)
	this.private.Validate()
	
	if(this.private.data[ID] == nil) then
		lib.chat.error("trying to access quest before adding it: "..ID)
		return
	end
	
	return this.private.data[ID]
end
