if(ThingsToDo2 == nil) then
	ThingsToDo2 = {}
end
local ttd2 = ThingsToDo2

if(ttd2.lib == nil) then
	ttd2.lib = {}
end
local lib = ttd2.lib

if(ttd2.option == nil) then
	ttd2.option = {}
end
local option = ttd2.option

if(ttd2.mod == nil) then
	ttd2.mod = {}
end
local mod = ttd2.mod

--this module
ttd2.BFAKeystone = {}
local this = ttd2.BFAKeystone
this.Name = "[BFA] Keystone"

this.private = { set = false }
function this.private.Validate()
	if(this.private.set == false) then
		this.private.Init()
		this.private.set = true
	end
	
	return
end

function this.private.Init()
	if(this.private.set == true) then
		return
	end
	
	this.private.KeystoneItemID = 158923
	this.private.MyLevel = lib.unit.GetPlayerLevel()
	
	--S2 Achievements
	this.private.Achievements = { [1] = 13448, [2] = 13449 }
	lib.achievement.AddByID(this.private.Achievements[1])
	lib.achievement.AddByID(this.private.Achievements[2])
	
	this.private.MaxRewardLevel = 10	--automatic calculation is not reliable
	this.private.CurrentCompletedLevel = nil
	this.private.OwnedKeystoneLink = nil
	this.private.IsRewardAvailable = nil

	lib.item.AddByID(this.private.KeystoneItemID)
	lib.item.UpdateByID(this.private.KeystoneItemID, true)
	
	return
end

function this.private.Update()
	this.private.Validate()
	
	local out = lib.chat.output
	out.ClearBuffer()
	
	if(this.private.MyLevel < 120) then
		--out.StoreTodo("Get to "..lib.chat.yellow("level 120").." to access M+ dungeons!")
		return
	end
	
	--update keystone functions
	lib.keystone.Update()
	
	--test if last week reward is collected
	if(lib.keystone.IsRewardChestAvailable() == true) then
		out.StoreTodo("Collect your "..lib.chat.green("Reward-Chest").." for last week!")
	elseif(lib.keystone.GetLastWeekBest() ~= nil) then
		out.StoreCompleted("Reward-Chest for last week already collected.")
	else
		out.StoreDone("Reward-Chest for last week not available.")
	end
	
	--test current week M+ completed
	local DungeonCompleted = false
	local CurrentBest = lib.keystone.GetCurrentWeekBest()
	if(CurrentBest < this.private.MaxRewardLevel) then
		out.StoreTodo("Completed: "..lib.chat.yellow(CurrentBest)..", Reward caps at: "..lib.chat.green(this.private.MaxRewardLevel))
	else
		out.StoreCompleted("Completed: "..lib.chat.green(CurrentBest))
		DungeonCompleted = true
	end
	
	--link stone
	lib.item.UpdateByID(this.private.KeystoneItemID, true)
	if(lib.item.GetByID(this.private.KeystoneItemID).BagLink ~= nil) then
		--if you own a key, you will get pestered! You have been warned!
		if(DungeonCompleted == true) then
			out.StoreCompleted("Your Keystone: "..lib.item.GetByID(this.private.KeystoneItemID).BagLink)
		else
			out.StoreTodo("Your Keystone: "..lib.item.GetByID(this.private.KeystoneItemID).BagLink)
		end
	end
	
	--achievements
	for k,AchID in ipairs(this.private.Achievements) do
		lib.achievement.UpdateByID(AchID)
		local AchData = lib.achievement.GetByID(AchID)
		
		if(AchData.IsCompleted == true) then
			out.StoreCompleted(AchData.Link)
		elseif(AchData.IsPartiallyCompleted == true) then
			--if you have already one dungeon completed for the Ach, you probably want to know the details
			
			--print achievement link todo
			out.StoreTodo(AchData.Link)
				
			--print detailed list
			for _,crit in pairs(AchData.CriteriaByIndex) do
				if(crit.IsCompleted == true) then
					out.StoreCompleted(crit.Name)
				else
					out.StoreTodo(crit.Name)
				end
			end
		elseif(DungeonCompleted == true) then
			--you have done your +10 for the week and have not completed any part of the ach yet: you probably don't care about it
			--use "done" as the output status
			out.StoreDone(AchData.Link)
		else
			--you have not done your +10 for the week, you have not completed any part of the ach yet
			--use "todo" as the output status
			out.StoreTodo(AchData.Link)
		end
	end
	
	return
end

function this.CheckStatus()
	this.private.Validate()
	
	this.private.Update()
	if(lib.chat.output.GetNumLines() == 0) then
		return false
	end
	
	return true
end

function this.Script()
	this.private.Validate()
	
	this.private.Update()
	lib.chat.output.PrintModule(this.Name)
	
	return
end

mod[#mod+1] =
{
	CheckStatus	= this.CheckStatus,
	Text		= this.Name,
	Script		= this.Script
}
